@@ -23,20 +23,12 @@
 #include <QObject>
 #include <QUrl>
 
+#include "gstpointer.h"
 #include <gst/gstpipeline.h>
 #include <gst/gstmessage.h>
 
 namespace QGst { namespace Quick { class VideoSurface; } }
 
-template <typename T>
-struct GstPointerCleanup
-{
-    static inline void cleanup(T *pointer)
-    {
-        if (pointer) gst_object_unref (pointer);
-    }
-};
-
 class Device;
 class WebcamControl : public QObject
 {
@@ -46,6 +38,15 @@ class WebcamControl : public QObject
         virtual ~WebcamControl();
 
         void onBusMessage(GstMessage* msg);
+        void setMirrored(bool m) {
+            if (m != m_mirror) {
+                m_mirror = m;
+                updateSourceFilter();
+                Q_EMIT mirroredChanged(m);
+            }
+        }
+
+        bool mirrored() const { return m_mirror; }
 
     public Q_SLOTS:
         bool play();
@@ -60,6 +61,7 @@ class WebcamControl : public QObject
 
     Q_SIGNALS:
         void photoTaken(const QString &photoUrl);
+        void mirroredChanged(bool mirrored);
 
     private:
         void updateSourceFilter();
@@ -68,10 +70,11 @@ class WebcamControl : public QObject
         QString m_extraFilters;
         QString m_tmpVideoPath;
         QString m_currentDevice;
-        QScopedPointer<GstPipeline, GstPointerCleanup<GstPipeline> > m_pipeline;
-        QScopedPointer<GstElement, GstPointerCleanup<GstElement> > m_cameraSource;
+        GstPointer<GstPipeline> m_pipeline;
+        GstPointer<GstElement> m_cameraSource;
         QGst::Quick::VideoSurface* m_surface = nullptr;
         bool m_emitTaken = true;
+        bool m_mirror = true;
 };
 
 #endif // WEBCAMCONTROL_H
